@@ -39,14 +39,20 @@ pub struct Wrapper {
     pub driver_type: DriverType,
     pub sql: String,
     pub args: Vec<serde_json::Value>,
-    pub formats: HashMap<String, fn(arg: &str) -> String>,
+    pub formats: HashMap<String, String>,
+}
+
+macro_rules! push_sql {
+    ($i:expr,$($v:expr,)*) => {
+        $($i.push_str($v);)*
+    };
 }
 
 impl Debug for Wrapper {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let mut formats = HashMap::new();
         for (k, v) in &self.formats {
-            formats.insert(k.to_string(), v(k));
+            formats.insert(k.to_string(), v.clone());
         }
         f.debug_struct("Wrapper")
             .field("driver_type", &self.driver_type)
@@ -81,7 +87,7 @@ impl Wrapper {
         self
     }
 
-    pub fn set_formats(mut self, formats: HashMap<String, fn(arg: &str) -> String>) -> Self {
+    pub fn set_formats(mut self, formats: HashMap<String, String>) -> Self {
         self.formats = formats;
         self
     }
@@ -101,24 +107,32 @@ impl Wrapper {
 
     /// push sql,args into self
     pub fn push<T>(mut self, sql: &str, args: &[T]) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         let mut new_sql = sql.to_string();
         if self.driver_type.is_number_type() {
             let self_arg_len = self.args.len();
             for index in 0..args.len() {
-                let str = self.driver_type.stmt_convert(index);
+                let mut convert_column = String::new();
+                self.driver_type.stmt_convert(index,&mut convert_column);
+
+                let mut convert_column_new = String::new();
+                self.driver_type.stmt_convert(index + args.len(),&mut convert_column_new);
                 new_sql = new_sql.replace(
-                    str.as_str(),
-                    self.driver_type.stmt_convert(index + args.len()).as_str(),
+                    convert_column.as_str(),
+                    convert_column_new.as_str(),
                 );
             }
             for index in args.len()..self_arg_len {
-                let str = self.driver_type.stmt_convert(index);
+                let mut convert_column = String::new();
+                self.driver_type.stmt_convert(index,&mut convert_column);
+
+                let mut convert_column_new = String::new();
+                self.driver_type.stmt_convert(index + args.len(),&mut convert_column_new);
                 new_sql = new_sql.replace(
-                    str.as_str(),
-                    self.driver_type.stmt_convert(index + args.len()).as_str(),
+                    convert_column.as_str(),
+                    convert_column_new.as_str(),
                 );
             }
         }
@@ -140,8 +154,8 @@ impl Wrapper {
     ///  let arg = 1;
     ///  wrapper.do_if(true, |w| w.eq("id"))
     pub fn do_if<'s, F>(self, test: bool, method: F) -> Self
-    where
-        F: FnOnce(Self) -> Self,
+        where
+            F: FnOnce(Self) -> Self,
     {
         if test {
             return method(self);
@@ -154,8 +168,8 @@ impl Wrapper {
     ///  let arg = 1;
     ///  wrapper.do_if(true, |w| w.eq("id"),|w|w)
     pub fn do_if_else<'s, F>(self, test: bool, method_if: F, method_else: fn(Self) -> Self) -> Self
-    where
-        F: FnOnce(Self) -> Self,
+        where
+            F: FnOnce(Self) -> Self,
     {
         if test {
             return method_if(self);
@@ -174,8 +188,8 @@ impl Wrapper {
     ///             ], |w| w.eq("a", "default"))
     ///             ;
     pub fn do_match<'s, F>(self, cases: &[(bool, fn(Wrapper) -> Wrapper)], default: F) -> Self
-    where
-        F: FnOnce(Self) -> Self,
+        where
+            F: FnOnce(Self) -> Self,
     {
         for (test, case) in cases {
             if *test {
@@ -196,8 +210,8 @@ impl Wrapper {
     }
 
     pub fn set_args<T>(mut self, args: &[T]) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         let v = json!(args);
         if v.is_null() {
@@ -210,8 +224,8 @@ impl Wrapper {
     }
 
     pub fn push_arg<T>(mut self, arg: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         let v = json!(arg);
         self.args.push(v);
@@ -265,15 +279,14 @@ impl Wrapper {
 
     pub fn having(mut self, sql_having: &str) -> Self {
         self = self.and();
-        self.sql
-            .push_str(format!(" {} {} ", crate::sql::TEMPLATE.having.value, sql_having).as_str());
+        push_sql!(self.sql," ",crate::sql::TEMPLATE.having.value," ",sql_having," ",);
         self
     }
 
     /// arg: JsonObject or struct{} or map[String,**]
     pub fn all_eq<T>(mut self, arg: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
         let v = json!(arg);
@@ -300,15 +313,13 @@ impl Wrapper {
     }
 
     ///format column
-    pub fn do_format_column(&self, column: &str, data: String) -> String {
+    pub fn do_format_column(&self, column: &str, data: &mut String) {
         let source = self.formats.get(column);
         match source {
             Some(s) => {
-                return s(&data);
-            }
-            _ => {
-                return data;
+                *data = s.clone();
             }
+            _ => {}
         }
     }
 
@@ -316,30 +327,28 @@ impl Wrapper {
     /// for example:
     ///  eq("a",1) " a = 1 "
     pub fn eq<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} = {}",
-            column,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," = ",convert_column.as_str(),);
         self.args.push(json!(obj));
         self
     }
 
     /// not equal
     pub fn ne<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} <> {}",
-            column,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," <> ",convert_column.as_str(),);
         self.args.push(json!(obj));
         self
     }
@@ -361,11 +370,9 @@ impl Wrapper {
             .push_str(&crate::sql::TEMPLATE.order_by.left_right_space);
         for x in columns {
             if is_asc {
-                self.sql
-                    .push_str(format!("{} {}", x, crate::sql::TEMPLATE.asc.value).as_str());
+                push_sql!(self.sql,x," ",crate::sql::TEMPLATE.asc.value,);
             } else {
-                self.sql
-                    .push_str(format!("{} {}", x, crate::sql::TEMPLATE.desc.value).as_str());
+                push_sql!(self.sql,x," ",crate::sql::TEMPLATE.desc.value,);
             }
             if (index + 1) != len {
                 self.sql.push_str(",");
@@ -402,103 +409,104 @@ impl Wrapper {
 
     ///  sql:   column > obj
     pub fn gt<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} > {}",
-            column,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," > ", &convert_column.as_str(),);
         self.args.push(json!(obj));
         self
     }
     ///  sql:   column >= obj
     pub fn ge<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} >= {}",
-            column,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," >= ", &convert_column.as_str(),);
         self.args.push(json!(obj));
         self
     }
 
     ///  sql:   column < obj
     pub fn lt<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} < {}",
-            column,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," < ", &convert_column.as_str(),);
         self.args.push(json!(obj));
         self
     }
 
     ///  sql:   column <= obj
     pub fn le<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} <= {}",
-            column,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," <= ", &convert_column.as_str(),);
         self.args.push(json!(obj));
         self
     }
 
     pub fn between<T>(mut self, column: &str, min: T, max: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} {} {} {} {}",
-            column,
-            crate::sql::TEMPLATE.between.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len())),
-            crate::sql::TEMPLATE.and.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len() + 1))
-        ));
+
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.between.value," ", &convert_column.as_str(),);
+
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql," ",crate::sql::TEMPLATE.and.value," ", &convert_column.as_str(),);
+
         self.args.push(json!(min));
         self.args.push(json!(max));
         self
     }
 
     pub fn not_between<T>(mut self, column: &str, min: T, max: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
-        self.sql.push_str(&format!(
-            "{} {} {} {} {} {}",
-            column,
-            crate::sql::TEMPLATE.not.value,
-            crate::sql::TEMPLATE.between.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len())),
-            crate::sql::TEMPLATE.and.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len() + 1))
-        ));
+
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.not.value,crate::sql::TEMPLATE.between.value," ", &convert_column.as_str(),);
+
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql," ",crate::sql::TEMPLATE.and.value," ", &convert_column.as_str(),);
+
         self.args.push(json!(min));
         self.args.push(json!(max));
         self
     }
 
     pub fn like<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
         let v = json!(obj);
@@ -508,19 +516,19 @@ impl Wrapper {
         } else {
             v_str = format!("%{}%", v.to_string());
         }
-        self.sql.push_str(&format!(
-            "{} {} {}",
-            column,
-            crate::sql::TEMPLATE.like.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.like.value," ", &convert_column.as_str(),);
+
         self.args.push(json!(v_str));
         self
     }
 
     pub fn like_left<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
         let v = json!(obj);
@@ -530,19 +538,19 @@ impl Wrapper {
         } else {
             v_str = format!("%{}", v.to_string());
         }
-        self.sql.push_str(&format!(
-            "{} {} {}",
-            column,
-            crate::sql::TEMPLATE.like.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.like.value," ", &convert_column.as_str(),);
+
         self.args.push(json!(v_str));
         self
     }
 
     pub fn like_right<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
         let v = json!(obj);
@@ -552,19 +560,19 @@ impl Wrapper {
         } else {
             v_str = format!("{}%", v.to_string());
         }
-        self.sql.push_str(&format!(
-            "{} {} {}",
-            column,
-            crate::sql::TEMPLATE.like.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.like.value," ", &convert_column.as_str(),);
+
         self.args.push(json!(v_str));
         self
     }
 
     pub fn not_like<T>(mut self, column: &str, obj: T) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self = self.and();
         let v = json!(obj);
@@ -574,13 +582,10 @@ impl Wrapper {
         } else {
             v_str = format!("%{}%", v.to_string());
         }
-        self.sql.push_str(&format!(
-            "{} {} {} {}",
-            column,
-            crate::sql::TEMPLATE.not.value,
-            crate::sql::TEMPLATE.like.value,
-            self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()))
-        ));
+        let mut convert_column = String::new();
+        self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+        self.do_format_column(column, &mut convert_column);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.not.value," ",crate::sql::TEMPLATE.like.value," ", &convert_column.as_str(),);
         self.args.push(json!(v_str));
         self
     }
@@ -604,70 +609,62 @@ impl Wrapper {
 
     /// gen sql: * in (*,*,*)
     pub fn in_array<T>(mut self, column: &str, obj: &[T]) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         if obj.len() == 0 {
             return self;
         }
         self = self.and();
-        let mut sqls = String::with_capacity(obj.len() * 10);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.r#in.value," (",);
         for x in obj {
-            sqls.push_str(&format!(
-                " {} ",
-                self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()),)
-            ));
-            sqls.push_str(",");
+            let mut convert_column = String::new();
+            self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+            self.do_format_column(column, &mut convert_column);
+            push_sql!(self.sql," ",&convert_column.as_str()," ",);
+            self.sql.push_str(",");
             self.args.push(json!(x));
         }
-        sqls.pop();
-        self.sql.push_str(
-            format!(" {} {} ({}) ", column, crate::sql::TEMPLATE.r#in.value, sqls).as_str(),
-        );
+        self.sql.pop();
+        push_sql!(self.sql,")",);
         self
     }
 
     /// gen sql: * in (*,*,*)
     pub fn in_<T>(self, column: &str, obj: &[T]) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self.in_array(column, obj)
     }
 
     /// gen sql: * in (*,*,*)
     pub fn r#in<T>(self, column: &str, obj: &[T]) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
         self.in_array(column, obj)
     }
 
     pub fn not_in<T>(mut self, column: &str, obj: &[T]) -> Self
-    where
-        T: Serialize,
+        where
+            T: Serialize,
     {
+        if obj.len() == 0 {
+            return self;
+        }
         self = self.and();
-        let mut sqls = String::with_capacity(obj.len() * 10);
+        push_sql!(self.sql,column," ",crate::sql::TEMPLATE.not.value," ",crate::sql::TEMPLATE.r#in.value," (",);
         for x in obj {
-            sqls.push_str(&format!(
-                " {} ",
-                self.do_format_column(column, self.driver_type.stmt_convert(self.args.len()),)
-            ));
-            sqls.push_str(",");
+            let mut convert_column = String::new();
+            self.driver_type.stmt_convert(self.args.len(),&mut convert_column);
+            self.do_format_column(column, &mut convert_column);
+            push_sql!(self.sql," ",&convert_column.as_str()," ",);
+            self.sql.push_str(",");
             self.args.push(json!(x));
         }
-        sqls.pop();
-        self.sql.push_str(
-            format!(
-                "{} {} {} ({})",
-                column,
-                crate::sql::TEMPLATE.not.value,
-                crate::sql::TEMPLATE.r#in.value,
-                sqls
-            )
-            .as_str(),
-        );
+        self.sql.pop();
+        push_sql!(self.sql,")",);
         self
     }
 
@@ -735,8 +732,10 @@ impl Wrapper {
     /// for example:
     ///  limit(1) " limit 1 "
     pub fn limit(mut self, limit: u64) -> Self {
-        self.sql
-            .push_str(&format!(" {} {} ", crate::sql::TEMPLATE.limit.value, limit));
+        use std::fmt::Write;
+        push_sql!(self.sql," ",crate::sql::TEMPLATE.limit.value," ",);
+        self.sql.write_fmt(format_args!("{}", limit));
+        self.sql.push_str(" ");
         self
     }
 }
